from functools import reduce
from pathlib import Path

from pandas import DataFrame
from pandas.io.json._normalize import nested_to_record


class CsvRunTable:
    TOP_LEVEL_KEYS = ['datasetName', 'pid', 'scientificMetadata']

    def __init__(self, path: str):
        self._path = path

    def dump(self, datasets) -> None:
        data_frames = [self._create_row(each) for each in datasets]
        result = reduce(DataFrame.combine_first, data_frames,
                        DataFrame(columns=self._create_header(datasets)))
        filepath = Path(self._path)
        filepath.parent.mkdir(parents=True, exist_ok=True)
        result.to_csv(filepath)

    @staticmethod
    def _remove_prefix(text, prefix: str):
        if isinstance(text, str):
            return text[text.startswith(prefix) and len(prefix):]
        return text

    @staticmethod
    def _create_header(datasets: list):
        header = []
        for run in datasets:
            run_header = nested_to_record(run, sep=' ')
            for key in run_header.keys():
                if any(x in key for x in CsvRunTable.TOP_LEVEL_KEYS):
                    header.append(CsvRunTable._remove_prefix(key, 'scientificMetadata '))
        return list(dict.fromkeys(header))

    @staticmethod
    def _create_row(dataset):
        top_level_keys = ['datasetName', 'pid', 'scientificMetadata']
        header = []
        values = []
        unnested_dataset = nested_to_record(dataset, sep=' ')
        for key in unnested_dataset.keys():
            if any(x in key for x in top_level_keys):
                header.append(CsvRunTable._remove_prefix(key, 'scientificMetadata '))
                values.append(CsvRunTable._remove_prefix(unnested_dataset[key], '/'))
        row = dict(zip(header, values))
        return DataFrame(row, index=[row['pid']])
