import json
from datetime import datetime

from .proposal import Proposal


class Dataset:

    def __init__(self, proposal_id, run_id: str):
        self._proposal_id = proposal_id
        self._run_id = run_id
        self._pid = f"{proposal_id}/{run_id}"
        self._metadata = None

    @property
    def pid(self):
        return self._pid

    @property
    def metadata(self):
        return self._metadata

    def create_metadata(self, run_name: str, scientific_metadata: dict, datatype: str, source_folder: str) -> None:
        self._get_proposal()
        metadata = {
            "datasetName": f"{run_name}",
            "pid": f"{self._pid}",
            "principalInvestigator":
                f"{self.proposal.get('pi_firstname', 'NaN')} {self.proposal.get('pi_lastname', 'NaN')}",
            "owner": f"{self.proposal['firstname']} {self.proposal['lastname']}",
            "contactEmail": self.proposal['email'],
            "sourceFolder": source_folder,
            "proposalId": self.proposal['proposalId'],
            "creationTime": datetime.now().strftime("%Y-%m-%dT%H:%M:%S"),
            "type": datatype,
            "creationLocation": self.proposal['MeasurementPeriodList'][0]['instrument'],
            "ownerGroup": self.proposal['ownerGroup'],
            "accessGroups": self.proposal['accessGroups'],
            "scientificMetadata": scientific_metadata
        }
        metadata["scientificMetadata"]["runNumber"] = {"value": self._run_id}
        self._metadata = json.dumps(metadata, indent=4)

    def update_metadata(self, scientific_metadata: dict) -> None:
        metadata = {
            "pid": f"/{self._pid}",  # '/' scicat implementation to accept http patch
            "scientificMetadata": scientific_metadata
        }
        metadata["scientificMetadata"]["runNumber"] = {"value": self._run_id}
        self._metadata = json.dumps(metadata, indent=4)

    def _get_proposal(self) -> None:
        self.proposal = Proposal(self._proposal_id).proposal
